package com.yaoyicloud.template;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import cn.hutool.core.util.StrUtil;
import com.deepoove.poi.data.NumberingFormat;
import com.deepoove.poi.data.NumberingRenderData;
import com.deepoove.poi.data.ParagraphRenderData;
import com.deepoove.poi.data.PictureType;
import com.deepoove.poi.data.Pictures;
import com.deepoove.poi.data.Texts;
import com.deepoove.poi.plugin.table.LoopRowTableRenderPolicy;
import com.yaoyicloud.config.FilerepoProperties;
import com.yaoyicloud.constant.enums.ReportType;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Component;
import com.deepoove.poi.XWPFTemplate;
import com.deepoove.poi.config.Configure;
import com.deepoove.poi.config.ConfigureBuilder;
import com.deepoove.poi.policy.RenderPolicy;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
@Slf4j
@Component
@RequiredArgsConstructor
public class PlatformCompanyReport extends AbstractReportExporter {


    private final FilerepoProperties filerepoProperties;

    @Override
    protected String getReportImagePath() {
        return filerepoProperties.getReportImagePath();
    }



    @Override
    protected String getTemplate(ReportType reportType, String basePath) {
        String reportTemplateFileName = getReportTemplateFileName(reportType);
        return basePath + File.separator + reportTemplateFileName;
    }

    @Override
    protected void bindConfigure(ConfigureBuilder builder, Map<String, Object> data) {
        LoopRowTableRenderPolicy hackLoopTableRenderPolicy = new LoopRowTableRenderPolicy();
        Object checkItemScores = data.get("checkItemScores");
        if (checkItemScores != null) {
            RenderPolicy scoreRenderPolicy = this.getScoreRenderPolicy();
            builder.bind("checkItemScores", scoreRenderPolicy).bind("basicInfoChecks", hackLoopTableRenderPolicy)
                    .bind("businessAbnormals", hackLoopTableRenderPolicy).bind("dishonestPersons", hackLoopTableRenderPolicy)
                    .bind("penaltyRecords", hackLoopTableRenderPolicy).bind("taxPenalties", hackLoopTableRenderPolicy)
                    .bind("severeViolations", hackLoopTableRenderPolicy).bind("questionnaireItems", hackLoopTableRenderPolicy)
                    .bind("indicators", hackLoopTableRenderPolicy);
        }
        builder.bind("basicInfoChecks", hackLoopTableRenderPolicy)
                .bind("businessAbnormals", hackLoopTableRenderPolicy).bind("dishonestPersons", hackLoopTableRenderPolicy)
                .bind("penaltyRecords", hackLoopTableRenderPolicy).bind("taxPenalties", hackLoopTableRenderPolicy)
                .bind("severeViolations", hackLoopTableRenderPolicy).bind("questionnaireItems", hackLoopTableRenderPolicy)
                .bind("indicators", hackLoopTableRenderPolicy);
    }

    @Override
    protected String imagePath() {
        return filerepoProperties.getReportImagePath();
    }


    @Override
    protected void renderTemplate(String reportTempWordFile, String templatePath, ConfigureBuilder builder,
        Map<String, Object> data) throws IOException {
        Configure config = builder.build();
        XWPFTemplate template = XWPFTemplate.compile(templatePath, config).render(data);
        template.writeToFile(reportTempWordFile);
        template.close();
    }



    @Override
    protected String getBasicPath() throws IOException {
        return filerepoProperties.getBasePath();
    }

    /**
     * 获取模板文件路径
     *
     * @param reportType
     * @return
     */
    public String getReportTemplateFileName(ReportType reportType) {
        return reportType.getType().toLowerCase() + "_template.docx";
    }

    @Override
    protected List<String> getFontPaths() {
        return Arrays.asList(filerepoProperties.getSourceHanSansCnFontMediumPath());
    }

    @Override
    public Map<String, Object> processData(Map<String, Object> rawData) throws Exception {
        Map<String, Object> processedData = new HashMap<>(rawData);

        String docurl1 = (String) processedData.get("docurl1");
        if (StrUtil.isNotBlank(docurl1)) {
            processedData.put("docurl1", Texts.of(docurl1).link(docurl1).create());
        }


        if (processedData.containsKey("docurl1Text")) {
            int i = 2;
            while (processedData.containsKey("docurl" + i)) {
                String url = (String) processedData.get("docurl" + i);
                if (StrUtil.isNotBlank(url)) {
                    processedData.put("docurl" + i, Texts.of(url).link(url).create());
                }
                i++;
            }
        }

        List<Map<String, Object>> imageDataList = (List<Map<String, Object>>) processedData.get("relatedEntitiesImage");
        List<ParagraphRenderData> paragraphImages = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(imageDataList)) {
            for (Map<String, Object> imageData : imageDataList) {
                ParagraphRenderData paragraphRenderData = new ParagraphRenderData();

                if (imageData.containsKey("base64")) {
                    String base64 = (String) imageData.get("base64");

                    String fileExtension = (String) imageData.get("fileExtension");
                    PictureType pictureType = PictureType.PNG; // 默认PNG
                    if (fileExtension != null) {
                        if (fileExtension.equalsIgnoreCase(".jpg") || fileExtension.equalsIgnoreCase(".jpeg")) {
                            pictureType = PictureType.JPEG;
                        } else if (fileExtension.equalsIgnoreCase(".gif")) {
                            pictureType = PictureType.GIF;
                        } else if (fileExtension.equalsIgnoreCase(".bmp")) {
                            pictureType = PictureType.BMP;
                        } else if (fileExtension.equalsIgnoreCase(".png")) {
                            pictureType = PictureType.PNG;
                        } else if (fileExtension.equalsIgnoreCase(".tiff")) {
                            pictureType = PictureType.TIFF;
                        }
                    }
                    paragraphRenderData.addPicture(
                            Pictures.ofBase64(base64, pictureType)
                                    .size((Integer) imageData.get("width"), (Integer) imageData.get("height"))
                                    .create());
                }
                paragraphImages.add(paragraphRenderData);
            }
            NumberingFormat numberingFormat = new NumberingFormat(1, "");
            NumberingRenderData numberedImages = new NumberingRenderData(numberingFormat, paragraphImages);
            processedData.put("relatedEntitiesImage", numberedImages);
        }
        // Process other specific links
        processLink(processedData, "socialSecurityDetailsLink");
        processLink(processedData, "taxDeclarationImage");
        processLink(processedData, "financialInfoLink");

        // Process image lists
        processImageListToNumberedRenderData("businessLicenseImage", processedData);
        processImageListToNumberedRenderData("bankCertificateImage", processedData);
        processImageListToNumberedRenderData("isoCertificationImage", processedData);
        processImageListToNumberedRenderData("securityLevelCertification", processedData);
        processImageListToNumberedRenderData("otherCertifications", processedData);
        processImageListToNumberedRenderData("organizationalStructureImage", processedData);
        processImageListToNumberedRenderData("taxPaymentCertificateImage", processedData);
        processImageListToNumberedRenderData("nsxydjwj", processedData);
        processImageListToNumberedRenderData("interestConflict", processedData);
        processImageListToNumberedRenderData("disanfangcangzhao", processedData);
        processImageListToNumberedRenderData("overallGuaranteeStatement", processedData);

        return processedData;
    }
}
